<?php
/*
 * © Copyright 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS 
 * 
 * iHRIS is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * The cadre pool form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2008 IntraHealth International, Inc. 
 * @since v3.1.0
 * @version v3.1.0
 */

/**
 * The cadre pool form.
 * @package iHRIS
 * @subpackage Plan
 * @access public
 */
class iHRIS_Pool extends iHRIS_YearlyList {
    /**
     * Get the field names that are yearly inputs
     * @returns array of string
     */ 
    public function yearlyInputs() {
        return  array('projected_rural','projected_urban');
    }



    public function getProjectedRuralHWs($year) {
        $this->projectRural(); 
        $fieldObj = $this->getField('projected_rural');
        if (!$fieldObj instanceof I2CE_FormField_ASSOC_INT) {
            I2CE::raiseError("No projected_rural");
            return 0;
        }
        if (!$fieldObj->keyIsSet($year)) {
            I2CE::raiseError("No projected_rural for year $year");
            return 0;
        }
        return max(0,$fieldObj->getValueOfKey($year));        
    }

    public function getProjectedUrbanHWs($year) {
        $this->projectUrban(); 
        $fieldObj = $this->getField('projected_urban');
        if (!$fieldObj instanceof I2CE_FormField_ASSOC_INT) {
            return 0;
        }
        if (!$fieldObj->keyIsSet($year)) {
            return 0;
        }
        return max(0,$fieldObj->getValueOfKey($year));
        
    }

    protected $projection = null;

    protected function ensureProjection() {
        if ($this->projection === null) {
            $this->projection = I2CE_FormFactory::instance()->createContainer($this->getParent());
            if ($this->projection instanceof iHRIS_Projection) {
                $this->projection->populate();
            } else {
                $this->projection = false;
            }
        }
        return $this->projection instanceof iHRIS_Projection;
    }

    protected function projectPool($field) {
        if (!$this->ensureProjection()) {
            return ;
        }
        $start = $this->projection->getStartYear();
        $duration = $this->projection->getDuration();
        if ($start === false || $duration == false) {
            return ;
        }
        $end = $start + $duration -1;

        if ($start > $end) {
            return;
        }
        $fieldObj = $this->getField('projected_' . $field);
        if (!$fieldObj instanceof I2CE_FormField_ASSOC_INT) {
            return;
        }
        $percent =   (((float) $this->percent) / 100.0);
        $all = 'approved_' . $field;
        $allocated = $this->$all;
        $fall = 'filled_' . $field;
        $fill= $this->$fall;


        if ($end == $start)  {
            if (!$fieldObj->keyIsSet($start)) {
                $fieldObj->setValueOfKey($start,$fill);
            }
            return;
        }
        $slope = (( $percent * ((float) $allocated)) - ((float) $fill)) / ((float) $end - (float)$start );
        
        for ($year = $start; $year <= $end; $year++) {
            if ($fieldObj->keyIsSet($year)) {
                continue;
            }
            $projection =((int) ( $slope * ($year - $start) + $fill));
            $fieldObj->setValueOfKey( $year,(int) $projection);
        }
    }


    protected $projected = array();
    public function projectRural() {
        if (!array_key_exists('rural',$this->projected) || !$this->projected['rural']) {
            $this->projectPool('rural');
        }
        $this->projected['rural'] = true;
    }
    public function projectUrban() {
        if (!array_key_exists('urban',$this->projected) || !$this->projected['urban']) {
            $this->projectPool('urban');
        }
        $this->projected['urban'] = true;
    }


    public function hasSupplyData() {
        $this->populateChildren( array( "supply_data" ));
        return (array_key_exists( 'supply_data', $this->children )  && is_array($this->children['supply_data']) && count($this->children['supply_data']) >0);
    }

    public function getPoolSize($start_year,$year_offset) {
        $this->populateChildren( array( "supply_data" ));

        if ( !array_key_exists( 'supply_data', $this->children ) || !is_array($this->children['supply_data']) || count($this->children['supply_data']) == 0) {
            return 0;
        }
        reset($this->children['supply_data']);
        $supplyObj = current($this->children['supply_data']);
        $size = $supplyObj->getField('supply');
        if (!$size instanceof I2CE_FormField_INT || !$size->issetValue()) {
            return 0;
        }
        return max(0,$size->getValue());
    }


    public function getSalary($start_year, $year_offset, $inflation_rate) {
        $base_salary = $this->getBaseSalary();
        return   ( (float) $base_salary) *  pow(  1.0+ (((float)$inflation_rate)/100),$year_offset) ;
    }

    public function getBaseSalary() {
        $this->populateChildren( array( "supply_data" ));

        if ( !array_key_exists( 'supply_data', $this->children ) || !is_array($this->children['supply_data']) || count($this->children['supply_data']) == 0) {
            return 0;
        }
        reset($this->children['supply_data']);
        $supplyObj = current($this->children['supply_data']);
        $salary = $supplyObj->getField('avg_salary');
        if (!$salary instanceof I2CE_FormField_FLOAT || !$salary->issetValue()) {
            return 0;
        }
        return max(0,$salary->getValue());
    }

}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
